%
%
%  Copy Filament to Next Timepoint
%
%  Copyright Bitplane BPI 2014
%  Matthew Gastinger, Ph.D
%  Application Support Scientist
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Copy Filament to Next Timepoint" icon="Matlab">
%          <Command>MatlabXT::XT_MJG_FilamentCopytoNextTimepoint(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Copy Filament to Next Timepoint" icon="Matlab">
%            <Command>MatlabXT::XT_MJG_FilamentCopytoNextTimepoint(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   This XTension will identify the filament in the last timepoint, copy it
%   and paste it into the next timepoint.  All other previoue filaments at
%   other timepoints will be copied into the new Filaments object.
%

function XT_MJG_FilamentCopytoNextTimepoint(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

% search the filament if not previously selected
vSurpassScene = vImarisApplication.GetSurpassScene;
if ~vFactory.IsFilaments(vFilaments)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vFactory.IsFilaments(vDataItem)
            vFilaments = vFactory.ToFilaments(vDataItem);
            break;
        end
    end
    % did we find the filament?
    if isequal(vFilaments, [])
        msgbox('Please create some filament!');
        return;
    end
end
vNumberOfFilaments=vFilaments.GetNumberOfFilaments;
vRGBA = vFilaments.GetColorRGBA
vNewFilament=vImarisApplication.GetFactory.CreateFilaments;
%%
%Identify Time Index for all Filament and put into Matrix
vTimeIndexFinal=[];
for s=0:vNumberOfFilaments-1
    vTimeIndex = vFilaments.GetTimeIndex(s);
    vTimeIndexFinal =[vTimeIndexFinal;vTimeIndex];
end
maxTime=max(vTimeIndexFinal);
%%
%Copy existing filament to NewFilament object
for c=0:vNumberOfFilaments-1
    CurrentFilamentTimeIndex = vFilaments.GetTimeIndex(c)
     if maxTime >= CurrentFilamentTimeIndex 
        aRadii = vFilaments.GetRadii(c);
        aPositions = vFilaments.GetPositionsXYZ(c);
        count =numel(aRadii);
        aType = zeros(count,1); %dendrite structure
        aEdges = vFilaments.GetEdges(c);
        aTime = CurrentFilamentTimeIndex;
        vNewFilament.AddFilament(aPositions, aRadii, aType, aEdges, aTime);
    end
end
%%
%Go through each filament, identify which timeIndex, and only copy those in
%from the last traced filament time point
for c=0:vNumberOfFilaments-1
    CurrentFilamentTimeIndex = vFilaments.GetTimeIndex(c)
     if maxTime == CurrentFilamentTimeIndex 
        aRadii = vFilaments.GetRadii(c);
        aPositions = vFilaments.GetPositionsXYZ(c);
        count =numel(aRadii);
        aType = zeros(count,1); %dendrite structure
        aEdges = vFilaments.GetEdges(c);
        aTime = CurrentFilamentTimeIndex+1;
        vNewFilament.AddFilament(aPositions, aRadii, aType, aEdges, aTime);
    end
end
vNewFilament.SetName(sprintf('next time', char(vFilaments.GetName)));
vNewFilament.SetName([char(vFilaments.GetName), ' next']);
vNewFilament.SetColorRGBA(vRGBA);
vFilaments.RemoveAllFilaments;
vFilaments.SetVisible(0);
vImarisApplication.GetSurpassScene.AddChild(vNewFilament, -1);




